# -*- coding: utf-8 -*-
"""
Functions to add citations recursivly for multiple ACS/Nature journals

"""

__authors__ = "Donna Löding, Alina Molkentin, Xinyi Tang, Judith Große, Malte Schokolowski"
__email__ = "cis-project2021@zbh.uni-hamburg.de"
__status__ = "Production"
#__copyright__ = ""
#__credits__ = ["", "", "", ""]
#__license__ = ""
#__version__ = ""
#__maintainer__ = ""


import sys  
from pathlib import Path
from os import error
sys.path.append("../../")

from input.publication import Publication
from verarbeitung.get_pub_from_input import get_pub

def create_global_lists_cit(input_nodes, input_edges, pub, search_height, search_height_max, test_var):
    '''
        :param input_nodes:         list of nodes from Processing
        :type input_nodes:          List[Publication]

        :param input_edges:         list of edges from Processing
        :type input_edges:          List[String, String]

        :param pub:                 Publication which citations will be added
        :type pub:                  Publication

        :param search_height:       current height to search for citations
        :type search_height_max:    int

        :param search_height_max:   maximum height to search for citations
        :type search_height_max:    int

        :param test_var:            variable to differenciate between test and url call
        :type test_var:             boolean

        function to create nodes and edges and call create_graph_structure_citations
    '''

    global nodes, edges
    nodes = input_nodes
    edges = input_edges

    return create_graph_structure_citations(pub, search_height, search_height_max, test_var)


def create_graph_structure_citations(pub, search_height, search_height_max, test_var):
    '''
        :param pub:                 publication which citations will be added
        :type pub:                  Publication

        :param search_height:       current height to search for citations
        :type search_height_max:    int

        :param search_height_max:   maximum height to search for citations
        :type search_height_max:    int

        :param test_var:            variable to differenciate between test and url call
        :type test_var:             boolean

        adds a node for every citing publication unknown
        adds edges to added citations
    '''

    citations_pub_obj_list = []
    for citation in pub.citations:
        not_in_nodes = True
        for node in nodes: # checks every citation for duplication 
            if (citation == node.doi_url):
                not_in_nodes = False
                break
        if (not_in_nodes):
            if (search_height < search_height_max): #checks if its a test and chooses input function accordingly
                citation_pub_obj = get_pub(citation, test_var)
                if (type(citation_pub_obj) != Publication):
                    print(pub)
                    continue 

                citation_pub_obj.group = "height"
                nodes.append(citation_pub_obj)
                edges.append([citation_pub_obj.doi_url,pub.doi_url])
                citations_pub_obj_list.append(citation_pub_obj)

        # adds only edge if citation already exists         
        elif [citation,pub.doi_url] not in edges:
            edges.append([citation,pub.doi_url])   
    return citations_pub_obj_list


def process_citations_rec(citations_pub_obj_list, search_height, search_height_max, test_var):  
    '''
        :param citations_pub_obj_list:  list of publications which citations will be added
        :type citations_pub_obj_list:   List[Publication]

        :param search_height:       current height to search for citations
        :type search_height_max:    int

        :param search_height_max:   maximum height to search for citations
        :type search_height_max:    int

        :param test_var:            variable to differenciate between test and url call
        :type test_var:             boolean

        recursive function to implement depth-first-search on citations
    '''

    # adds next level to nodes/edges
    for pub in citations_pub_obj_list:
        new_citation_pub_obj_list = create_graph_structure_citations(pub, search_height, search_height_max, test_var)   

        # If the maximum height has not yet been reached, calls function recursivly with increased height 
        if (search_height < search_height_max):
            process_citations_rec(new_citation_pub_obj_list, search_height+1, search_height_max, test_var)


def add_citations(input_nodes, input_edges, citations_pub_obj_list, search_height, search_height_max, test_var):
    '''
        :param input_nodes:             list of nodes from Processing
        :type input_nodes:              List[Publication]

        :param input_edges:             list of edges from Processing
        :type input_edges:              List[String, String]

        :param citations_pub_obj_list:  list of publications which citations will be added
        :type citations_pub_obj_list:   List[Publication]

        :param search_height:       current height to search for citations
        :type search_height_max:    int

        :param search_height_max:   maximum height to search for citations
        :type search_height_max:    int

        :param test_var:            variable to differenciate between test and url call
        :type test_var:             boolean

        function to call recursive depth-first-search of citations
    '''
    global nodes, edges
    nodes = input_nodes
    edges = input_edges

    process_citations_rec(citations_pub_obj_list, search_height, search_height_max, test_var)
    return(nodes, edges)